@@ -1,14 +1,17 @@
 from __future__ import annotations
 
+import dataclasses
 import logging
 from collections import defaultdict
 from dataclasses import dataclass
+from typing import Sequence
 
 from django.contrib import admin
 from django.contrib.admin.options import BaseModelAdmin, InlineModelAdmin
 from django.contrib.admin.utils import flatten_fieldsets
 from django.contrib.contenttypes.admin import GenericInlineModelAdmin
 from django.db import models
+from django.db.models import functions
 from django.db.models.fields.reverse_related import ForeignObjectRel
 from django.forms.models import _get_foreign_key
 from django.urls import reverse
@@ -16,9 +19,10 @@ from django.urls import reverse
 from .query import (
     ASC,
     DSC,
+    TYPES,
     BooleanFieldType,
-    FieldType,
     HTMLFieldType,
+    MetaFieldType,
     NumberFieldType,
     StringFieldType,
     TimeFieldType,
@@ -52,29 +56,208 @@ _AGG_MAP = {
 }
 
 
+_AGGREGATES = {
+    StringFieldType: ["count"],
+    NumberFieldType: ["average", "count", "max", "min", "std_dev", "sum", "variance"],
+    TimeFieldType: ["count"],  # average, min and max might be nice here but sqlite...
+    BooleanFieldType: ["average", "sum"],
+}
+
+
+_FUNC_MAP = {
+    "year": functions.ExtractYear,
+    "quarter": functions.ExtractQuarter,
+    "month": functions.ExtractMonth,
+    "day": functions.ExtractDay,
+    "week_day": functions.ExtractWeekDay,
+    "hour": functions.ExtractHour,
+    "minute": functions.ExtractMinute,
+    "second": functions.ExtractSecond,
+}
+
+if hasattr(functions, "ExtractIsoYear"):  # pragma: no branch
+    _FUNC_MAP.update(
+        {"iso_year": functions.ExtractIsoYear, "iso_week": functions.ExtractWeek}
+    )
+
+_FUNCTIONS = {
+    TimeFieldType: [
+        "year",
+        "quarter",
+        "month",
+        "day",
+        "week_day",
+        "hour",
+        "minute",
+        "second",
+    ]
+}
+
+
 def get_model_name(model, sep="."):
     return f"{model._meta.app_label}{sep}{model.__name__}"
 
 
+@dataclass
+class OrmBoundField:
+    orm_field: OrmBaseField = None
+    db_field: MetaFieldType = None
+    model_path: Sequence[str] = dataclasses.field(default_factory=list)
+    pretty_path: Sequence[str] = dataclasses.field(default_factory=list)
+
+    @property
+    def field_path(self):
+        return self.model_path + [self.db_field.name]
+
+    @property
+    def full_path(self):
+        # todo this is kinda ugly
+        if self.orm_field == self.db_field:
+            return self.field_path
+        return self.field_path + [self.orm_field.name]
+
+    @property
+    def full_path_str(self):
+        return "__".join(self.full_path)
+
+    @property
+    def field_path_str(self):
+        return "__".join(self.field_path)
+
+    @property
+    def model_name(self):
+        return self.db_field.model_name
+
+    @property
+    def concrete(self):
+        return self.orm_field.concrete
+
+    @property
+    def type_(self):
+        return self.orm_field.type_
+
+    @property
+    def aggregate(self):
+        return self.orm_field.aggregate
+
+    @property
+    def function(self):
+        return self.orm_field.function
+
+
 @dataclass
 class OrmModel:
     fields: dict
-    fks: dict
     admin: BaseModelAdmin = None
 
-
-@dataclass
-class OrmField:
-    type_: FieldType
-    concrete: bool
-    model_name: str
-    pretty_name: str
+    @property
+    def root(self):
+        return bool(self.admin)
 
 
-@dataclass
-class OrmFkField:
-    model_name: str
-    pretty_name: str
+class OrmBaseField:
+    def __init__(self, model_name, name, pretty_name):
+        self.model_name = model_name
+        self.name = name
+        self.pretty_name = pretty_name
+
+    type_ = None  # can't add
+    concrete = False  # can't sort or filter
+    rel_name = None  # can't expand
+
+    # internal
+    aggregate = None
+    function = None
+
+    def __repr__(self):  # pragma: no cover
+        params = [
+            self.model_name,
+            self.name,
+            self.pretty_name,
+            self.type_,
+            self.concrete,
+            self.rel_name,
+        ]
+        return f"{self.__class__.__name__}({', '.join(repr(p) for p in params)})"
+
+    def bind(self, previous):
+        previous = previous or OrmBoundField()
+        return OrmBoundField(
+            self, self, previous.model_path, previous.pretty_path + [self.pretty_name]
+        )
+
+
+class OrmFkField(OrmBaseField):
+    def __init__(self, model_name, name, pretty_name, rel_name):
+        super().__init__(model_name, name, pretty_name)
+        self.rel_name = rel_name
+
+    def bind(self, previous):
+        previous = previous or OrmBoundField()
+        return OrmBoundField(
+            self,
+            None,
+            previous.model_path + [self.name],
+            previous.pretty_path + [self.pretty_name],
+        )
+
+
+class OrmConcreteField(OrmBaseField):
+    concrete = True
+
+    def __init__(self, model_name, name, pretty_name, type_):
+        super().__init__(model_name, name, pretty_name)
+        self.type_ = type_
+        self.rel_name = (
+            type_.name if type_ in _AGGREGATES or type_ in _FUNCTIONS else None
+        )
+
+
+class OrmCalculatedField(OrmBaseField):
+    type_ = StringFieldType
+
+
+class OrmAdminField(OrmBaseField):
+    type_ = HTMLFieldType
+
+    def __init__(self, model_name):
+        super().__init__(model_name, _OPEN_IN_ADMIN, _OPEN_IN_ADMIN)
+
+
+class OrmAggregateField(OrmBaseField):
+    type_ = NumberFieldType
+    concrete = True
+
+    def __init__(self, model_name, name):
+        super().__init__(model_name, name, name)
+        self.aggregate = name
+
+    def bind(self, previous):
+        assert previous.db_field
+        return OrmBoundField(
+            self,
+            previous.db_field,
+            previous.model_path,
+            previous.pretty_path + [self.pretty_name],
+        )
+
+
+class OrmFunctionField(OrmBaseField):
+    type_ = NumberFieldType
+    concrete = True
+
+    def __init__(self, model_name, name):
+        super().__init__(model_name, name, name)
+        self.function = name
+
+    def bind(self, previous):
+        assert previous.db_field
+        return OrmBoundField(
+            self,
+            previous.db_field,
+            previous.model_path,
+            previous.pretty_path + [self.pretty_name],
+        )
 
 
 def _get_all_admin_fields(request):
@@ -87,24 +270,25 @@ def _get_all_admin_fields(request):
             if not isinstance(admin, InlineModelAdmin) or hasattr(admin.model, f):
                 yield f
 
-    def visible(modeladmin, request):
-        if modeladmin.has_change_permission(request):
+    def visible(model_admin, request):
+        if model_admin.has_change_permission(request):
             return True
-        if hasattr(modeladmin, "has_view_permission"):
-            return modeladmin.has_view_permission(request)
+        if hasattr(model_admin, "has_view_permission"):
+            return model_admin.has_view_permission(request)
         else:
             return False  # pragma: no cover  Django < 2.1 compat
 
     all_admin_fields = defaultdict(set)
     model_admins = {}
-    for model, modeladmin in admin.site._registry.items():
-        model_admins[model] = modeladmin
-        if visible(modeladmin, request):
-            all_admin_fields[model].update(from_fieldsets(modeladmin, True))
+    for model, model_admin in admin.site._registry.items():
+        model_admins[model] = model_admin
+        if visible(model_admin, request):
+            all_admin_fields[model].update(from_fieldsets(model_admin, True))
+            all_admin_fields[model].update(model_admin.get_list_display(request))
             all_admin_fields[model].add(_OPEN_IN_ADMIN)
 
             # check the inlines, these are already filtered for access
-            for inline in modeladmin.get_inline_instances(request):
+            for inline in model_admin.get_inline_instances(request):
                 if not isinstance(inline, GenericInlineModelAdmin):  # pragma: no branch
                     if inline.model not in model_admins:  # pragma: no branch
                         model_admins[inline.model] = inline
@@ -117,13 +301,13 @@ def _get_all_admin_fields(request):
     for fields in all_admin_fields.values():
         fields.add("id")
         fields.discard("pk")
+        fields.discard("__str__")
 
     return model_admins, all_admin_fields
 
 
 def _get_fields_for_model(model, model_admins, admin_fields):
     fields = {}
-    fks = {}
 
     model_name = get_model_name(model)
     model_fields = {f.name: f for f in model._meta.get_fields()}
@@ -131,26 +315,20 @@ def _get_fields_for_model(model, model_admins, admin_fields):
     for field_name in admin_fields[model]:
         field = model_fields.get(field_name)
         if field_name == _OPEN_IN_ADMIN:
-            fields[_OPEN_IN_ADMIN] = OrmField(
-                type_=HTMLFieldType,
-                concrete=False,
-                model_name=model_name,
-                pretty_name=_OPEN_IN_ADMIN,
-            )
+            fields[_OPEN_IN_ADMIN] = OrmAdminField(model_name=model_name)
         elif isinstance(field, (ForeignObjectRel, models.ManyToManyField)):
             pass  # TODO 2many support
         elif isinstance(field, models.ForeignKey):
             if field.related_model in admin_fields:
-                fks[field_name] = OrmFkField(
-                    model_name=get_model_name(field.related_model),
+                fields[field_name] = OrmFkField(
+                    model_name=model_name,
+                    name=field_name,
                     pretty_name=field_name,
+                    rel_name=get_model_name(field.related_model),
                 )
         elif isinstance(field, type(None)):
-            fields[field_name] = OrmField(
-                type_=StringFieldType,
-                concrete=False,
-                model_name=model_name,
-                pretty_name=field_name,
+            fields[field_name] = OrmCalculatedField(
+                model_name=model_name, name=field_name, pretty_name=field_name
             )
         else:
             if field.__class__ in _FIELD_MAP:
@@ -163,27 +341,37 @@ def _get_fields_for_model(model, model_admins, admin_fields):
                     field_type = None
 
             if field_type:
-                fields[field_name] = OrmField(
-                    type_=field_type,
-                    concrete=True,
+                fields[field_name] = OrmConcreteField(
                     model_name=model_name,
+                    name=field_name,
                     pretty_name=field_name,
+                    type_=field_type,
                 )
             else:
                 logging.getLogger(__name__).warning(
                     f"{model.__name__}.{field_name} unsupported type {type(field).__name__}"
                 )
 
-    return OrmModel(fields=fields, fks=fks, admin=model_admins[model])
+    return OrmModel(fields=fields, admin=model_admins[model])
+
+
+def _get_fields_for_type(type_):
+    aggregates = {
+        a: OrmAggregateField(type_.name, a) for a in _AGGREGATES.get(type_, [])
+    }
+    functions = {f: OrmFunctionField(type_.name, f) for f in _FUNCTIONS.get(type_, [])}
+    return OrmModel({**aggregates, **functions})
 
 
 def get_models(request):
     model_admins, admin_fields = _get_all_admin_fields(request)
-    # {model: {"fields": {field_name, FieldType}, "fks": {field_name: model}}}
-    return {
+    models = {
         get_model_name(model): _get_fields_for_model(model, model_admins, admin_fields)
         for model in admin_fields
     }
+    types = {type_.name: _get_fields_for_type(type_) for type_ in TYPES.values()}
+
+    return {**models, **types}
 
 
 def _get_django_lookup(field_type, lookup):
@@ -214,38 +402,46 @@ def get_results(request, bound_query):
             negation = True
             lookup = lookup[4:]
 
-        filter_str = f"{filter_.path_str}__{_get_django_lookup(filter_.type_, lookup)}"
+        filter_str = f"{filter_.orm_bound_field.full_path_str}__{_get_django_lookup(filter_.orm_bound_field.type_, lookup)}"
         if negation:
             return qs.exclude(**{filter_str: filter_.parsed})
         else:
             return qs.filter(**{filter_str: filter_.parsed})
 
     def fmt(field, value):
-        if field.aggregate:
-            return NumberFieldType.format(value)
-        else:
-            return field.type_.format(value)
+        return field.orm_bound_field.type_.format(value)
 
     request.data_browser = True
 
     if not bound_query.fields:
         return []
 
-    normal_fields = [f for f in bound_query.fields if not f.aggregate]
+    normal_fields = [f for f in bound_query.fields if not f.orm_bound_field.aggregate]
+    calculated_fields = [
+        f for f in bound_query.fields if not f.orm_bound_field.concrete
+    ]
 
     admin = bound_query.orm_models[bound_query.model_name].admin
     qs = admin.get_queryset(request)
 
-    # filter normal fields
+    # functions
+    for field in bound_query.fields + bound_query.filters:
+        field = field.orm_bound_field
+        if field.function:
+            qs = qs.annotate(
+                **{field.full_path_str: _FUNC_MAP[field.function](field.field_path_str)}
+            )
+
+    # filter normal and function fields
     for filter_ in bound_query.valid_filters:
-        if not filter_.aggregate:
+        if not filter_.orm_bound_field.aggregate:
             qs = filter(qs)
 
     # no calculated fields we're going to early out using qs.values
-    if not bound_query.calculated_fields:
+    if not calculated_fields:
         # .values() is interpreted as all values, _ddb_dummy ensures there's always at least one
         qs = qs.values(
-            *[f.path_str for f in normal_fields],
+            *[f.orm_bound_field.full_path_str for f in normal_fields],
             _ddb_dummy=models.Value(1, output_field=models.IntegerField()),
         )
 
@@ -254,31 +450,35 @@ def get_results(request, bound_query):
 
     # aggregates
     for field in bound_query.fields + bound_query.filters:
+        field = field.orm_bound_field
         if field.aggregate:
             qs = qs.annotate(
-                **{field.path_str: _AGG_MAP[field.aggregate](field.field_path_str)}
+                **{field.full_path_str: _AGG_MAP[field.aggregate](field.field_path_str)}
             )
 
     # filter aggregate fields
     for filter_ in bound_query.valid_filters:
-        if filter_.aggregate:
+        if filter_.orm_bound_field.aggregate:
             qs = filter(qs)
 
     # sort
     sort_fields = []
     for field in bound_query.sort_fields:
         if field.direction is ASC:
-            sort_fields.append(field.path_str)
+            sort_fields.append(field.orm_bound_field.full_path_str)
         if field.direction is DSC:
-            sort_fields.append(f"-{field.path_str}")
+            sort_fields.append(f"-{field.orm_bound_field.full_path_str}")
     qs = qs.order_by(*sort_fields)
 
     # no calculated fields early out using qs.values
-    if not bound_query.calculated_fields:
+    if not calculated_fields:
         results = []
         for row in qs:
             results.append(
-                [fmt(field, row[field.path_str]) for field in bound_query.fields]
+                [
+                    fmt(field, row[field.orm_bound_field.full_path_str])
+                    for field in bound_query.fields
+                ]
             )
         return results
 
@@ -289,13 +489,13 @@ def get_results(request, bound_query):
 
     select_related = set()
     for field in bound_query.sort_fields:
-        select_related.update(ancestors(field.model_path))
+        select_related.update(ancestors(field.orm_bound_field.model_path))
     for filter_ in bound_query.valid_filters:
-        select_related.update(ancestors(filter_.model_path))
+        select_related.update(ancestors(filter_.orm_bound_field.model_path))
 
     prefetch_related = set()
     for field in normal_fields:
-        prefetch_related.update(ancestors(field.model_path))
+        prefetch_related.update(ancestors(field.orm_bound_field.model_path))
     prefetch_related -= select_related
 
     if select_related:
@@ -315,14 +515,14 @@ def get_results(request, bound_query):
     def lookup(obj, field):
         value = obj
 
-        if field.aggregate is None:
-            *parts, tail = field.path
+        if field.aggregate is None and field.function is None:
+            *parts, tail = field.full_path
             for part in parts:
                 value = getattr(value, part, None)
         else:
-            tail = field.path_str
+            tail = field.full_path_str
 
-        admin = bound_query.orm_models[field.orm_field.model_name].admin
+        admin = bound_query.orm_models[field.model_name].admin
         if field.concrete:
             return getattr(value, tail, None)
         elif tail == _OPEN_IN_ADMIN:
@@ -342,5 +542,10 @@ def get_results(request, bound_query):
 
     results = []
     for row in qs:
-        results.append([fmt(field, lookup(row, field)) for field in bound_query.fields])
+        results.append(
+            [
+                fmt(field, lookup(row, field.orm_bound_field))
+                for field in bound_query.fields
+            ]
+        )
     return results
