@@ -144,7 +144,6 @@ class StringFieldType(FieldType):
         "not_regex": "string",
         "is_null": "boolean",
     }
-    aggregates = ["count"]
 
 
 class NumberFieldType(FieldType):
@@ -158,7 +157,6 @@ class NumberFieldType(FieldType):
         "lte": "number",
         "is_null": "boolean",
     }
-    aggregates = ["average", "count", "max", "min", "std_dev", "sum", "variance"]
 
     @staticmethod
     def format(value):
@@ -170,7 +168,7 @@ class NumberFieldType(FieldType):
 
 
 class TimeFieldType(FieldType):
-    default_value = timezone.now().strftime("%Y-%m-%d %H:%M")
+    default_value = "now"
     lookups = {
         "equals": "time",
         "not_equals": "time",
@@ -180,10 +178,11 @@ class TimeFieldType(FieldType):
         "lte": "time",
         "is_null": "boolean",
     }
-    aggregates = ["count"]
 
     @staticmethod
     def parse(value):
+        if value.lower().strip() == "now":
+            return timezone.now()
         return timezone.make_aware(dateutil.parser.parse(value))
 
     @staticmethod
@@ -194,13 +193,11 @@ class TimeFieldType(FieldType):
 class HTMLFieldType(FieldType):
     default_value = None
     lookups = {}
-    aggregates = []
 
 
 class BooleanFieldType(FieldType):
     default_value = True
     lookups = {"equals": "boolean", "not_equals": "boolean", "is_null": "boolean"}
-    aggregates = ["average", "sum"]
 
     @staticmethod
     def parse(value):
@@ -226,57 +223,30 @@ TYPES = {
 
 
 class BoundFieldMixin:
-    @property
-    def model_path_str(self):
-        return "__".join(self.model_path)
-
-    @property
-    def field_path(self):
-        return (self.model_path or []) + [self.name]
-
-    @property
-    def field_path_str(self):
-        return "__".join(self.field_path)
-
     @property
     def path(self):
-        return self.field_path + ([self.aggregate] if self.aggregate else [])
+        return self.orm_bound_field.full_path
 
     @property
-    def path_str(self):
-        return "__".join(self.path)
+    def pretty_path(self):
+        return self.orm_bound_field.pretty_path
 
 
 @dataclass
 class BoundFilter(BoundFieldMixin):
-    model_path: Sequence[str]
-    name: str
-    aggregate: Optional[str]
-    pretty_path: Sequence[str]
+    orm_bound_field: Any
     lookup: str
     value: str
-    type_: FieldType
 
     @classmethod
-    def bind(cls, path, name, aggregate, pretty_path, query_filter, orm_field):
-        return cls(
-            path,
-            name,
-            aggregate,
-            pretty_path,
-            query_filter.lookup,
-            query_filter.value,
-            orm_field.type_,
-        )
+    def bind(cls, orm_bound_field, query_filter):
+        return cls(orm_bound_field, query_filter.lookup, query_filter.value)
 
     def __post_init__(self):
         self.parsed = None
         self.err_message = None
 
-        if self.aggregate:
-            lookups = NumberFieldType.lookups
-        else:
-            lookups = self.type_.lookups
+        lookups = self.orm_bound_field.type_.lookups
         if self.lookup not in lookups:
             self.err_message = f"Bad lookup '{self.lookup}' expected {lookups}"
         else:
@@ -290,104 +260,52 @@ class BoundFilter(BoundFieldMixin):
 
 @dataclass
 class BoundField(BoundFieldMixin):
-    model_path: Sequence[str]
-    name: str
-    aggregate: Optional[str]
-    pretty_path: Sequence[str]
+    orm_bound_field: Any
     direction: Optional[str]
     priority: Optional[int]
-    orm_field: Any  # orm.OrmField
-
-    def __post_init__(self):
-        self.type_ = self.orm_field.type_
-        self.concrete = self.orm_field.concrete
 
     @classmethod
-    def bind(cls, path, name, aggregate, pretty_path, query_field, orm_field):
-        direction = query_field.direction if orm_field.concrete else None
-        priority = query_field.priority if orm_field.concrete else None
-        return cls(path, name, aggregate, pretty_path, direction, priority, orm_field)
+    def bind(cls, orm_bound_field, query_field):
+        concrete = orm_bound_field.concrete
+        direction = query_field.direction if concrete else None
+        priority = query_field.priority if concrete else None
+        return cls(orm_bound_field, direction, priority)
 
 
 class BoundQuery:
     def __init__(self, query, orm_models):
-        # orm_models = {model_name: {"fields": {field_name, FieldType}, "fks": {field_name: model_name}}}
-        def get_path(parts, model_name):
-            pretty_parts = []
+        def get_orm_field(parts):
+            model_name = query.model_name
+            orm_bound_field = None
             for part in parts:
-                fk_field = orm_models[model_name].fks.get(part)
-                if fk_field is None:
-                    return None, None
-                pretty_parts.append(fk_field.pretty_name)
-                model_name = fk_field.model_name
-            return model_name, pretty_parts
-
-        def get_orm_field(path):
-            # path__field
-            *model_path, field_name = path
-            model_name, pretty_parts = get_path(model_path, query.model_name)
-            if model_name:
-                orm_field = orm_models[model_name].fields.get(field_name)
-                if orm_field:
-                    return (
-                        orm_field,
-                        model_path,
-                        field_name,
-                        None,
-                        pretty_parts + [orm_field.pretty_name],
-                    )
-
-            # path__field__aggregate
-            if len(path) >= 2:
-                *model_path, field_name, aggregate = path
-                model_name, pretty_parts = get_path(model_path, query.model_name)
-                if model_name:
-                    orm_field = orm_models[model_name].fields.get(field_name)
-                    if orm_field and aggregate in orm_field.type_.aggregates:
-                        return (
-                            orm_field,
-                            model_path,
-                            field_name,
-                            aggregate,
-                            pretty_parts + [orm_field.pretty_name, aggregate],
-                        )
-            return None, None, None, None, None
+                orm_field = orm_models[model_name].fields.get(part)
+                if orm_field is None:
+                    return None
+                orm_bound_field = orm_field.bind(orm_bound_field)
+                model_name = orm_field.rel_name
+            if not orm_bound_field.type_:
+                return None
+            return orm_bound_field
 
         self.model_name = query.model_name
         self.orm_models = orm_models
 
         self.fields = []
         for query_field in query.fields:
-            orm_field, path, name, aggregate, pretty_path = get_orm_field(
-                query_field.path
-            )
-            if orm_field:
-                self.fields.append(
-                    BoundField.bind(
-                        path, name, aggregate, pretty_path, query_field, orm_field
-                    )
-                )
+            orm_bound_field = get_orm_field(query_field.path)
+            if orm_bound_field:
+                self.fields.append(BoundField.bind(orm_bound_field, query_field))
 
         self.filters = []
         for query_filter in query.filters:
-            orm_field, path, name, aggregate, pretty_path = get_orm_field(
-                query_filter.path
-            )
-            if orm_field:
-                self.filters.append(
-                    BoundFilter.bind(
-                        path, name, aggregate, pretty_path, query_filter, orm_field
-                    )
-                )
+            orm_bound_field = get_orm_field(query_filter.path)
+            if orm_bound_field:
+                self.filters.append(BoundFilter.bind(orm_bound_field, query_filter))
 
     @property
     def sort_fields(self):
         return sorted((f for f in self.fields if f.direction), key=lambda f: f.priority)
 
-    @property
-    def calculated_fields(self):
-        return {f.path_str for f in self.fields if not f.concrete}
-
     @property
     def valid_filters(self):
         return [f for f in self.filters if f.is_valid]
