@@ -493,6 +493,12 @@ snapshots["test_query_html context"] = {
                         "prettyName": "name",
                         "type": "string",
                     },
+                    "only_in_list_view": {
+                        "concrete": True,
+                        "model": "string",
+                        "prettyName": "only_in_list_view",
+                        "type": "string",
+                    },
                     "onsale": {
                         "concrete": True,
                         "model": "boolean",
@@ -525,6 +531,7 @@ snapshots["test_query_html context"] = {
                     "default_sku",
                     "is_onsale",
                     "name",
+                    "only_in_list_view",
                     "onsale",
                     "producer",
                     "size",
@@ -584,9 +591,67 @@ snapshots["test_query_html context"] = {
                         "model": None,
                         "prettyName": "count",
                         "type": "number",
-                    }
+                    },
+                    "day": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "day",
+                        "type": "number",
+                    },
+                    "hour": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "hour",
+                        "type": "number",
+                    },
+                    "minute": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "minute",
+                        "type": "number",
+                    },
+                    "month": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "month",
+                        "type": "number",
+                    },
+                    "quarter": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "quarter",
+                        "type": "number",
+                    },
+                    "second": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "second",
+                        "type": "number",
+                    },
+                    "week_day": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "week_day",
+                        "type": "number",
+                    },
+                    "year": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "year",
+                        "type": "number",
+                    },
                 },
-                "sortedFields": ["count"],
+                "sortedFields": [
+                    "count",
+                    "day",
+                    "hour",
+                    "minute",
+                    "month",
+                    "quarter",
+                    "second",
+                    "week_day",
+                    "year",
+                ],
             },
         },
         "baseUrl": "/data_browser/",
@@ -675,7 +740,7 @@ snapshots["test_query_html context"] = {
             },
             "time": {
                 "defaultLookup": "equals",
-                "defaultValue": "redacted",
+                "defaultValue": "now",
                 "lookups": {
                     "equals": {"type": "time"},
                     "gt": {"type": "time"},
@@ -709,24 +774,14 @@ snapshots["test_query_html context"] = {
                 "sort": None,
             },
         ],
+        "filterErrors": [None, None],
         "filters": [
-            {
-                "errorMessage": None,
-                "lookup": "lt",
-                "path": ["size"],
-                "prettyPath": ["size"],
-                "value": "2",
-            },
-            {
-                "errorMessage": None,
-                "lookup": "gt",
-                "path": ["id"],
-                "prettyPath": ["id"],
-                "value": "0",
-            },
+            {"lookup": "lt", "path": ["size"], "prettyPath": ["size"], "value": "2"},
+            {"lookup": "gt", "path": ["id"], "prettyPath": ["id"], "value": "0"},
         ],
         "model": "tests.Product",
         "results": [],
+        "version": "redacted",
     },
     "sentryDsn": None,
 }
@@ -826,9 +881,67 @@ snapshots["test_query_html_no_perms context"] = {
                         "model": None,
                         "prettyName": "count",
                         "type": "number",
-                    }
+                    },
+                    "day": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "day",
+                        "type": "number",
+                    },
+                    "hour": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "hour",
+                        "type": "number",
+                    },
+                    "minute": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "minute",
+                        "type": "number",
+                    },
+                    "month": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "month",
+                        "type": "number",
+                    },
+                    "quarter": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "quarter",
+                        "type": "number",
+                    },
+                    "second": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "second",
+                        "type": "number",
+                    },
+                    "week_day": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "week_day",
+                        "type": "number",
+                    },
+                    "year": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "year",
+                        "type": "number",
+                    },
                 },
-                "sortedFields": ["count"],
+                "sortedFields": [
+                    "count",
+                    "day",
+                    "hour",
+                    "minute",
+                    "month",
+                    "quarter",
+                    "second",
+                    "week_day",
+                    "year",
+                ],
             },
         },
         "baseUrl": "/data_browser/",
@@ -905,7 +1018,7 @@ snapshots["test_query_html_no_perms context"] = {
             },
             "time": {
                 "defaultLookup": "equals",
-                "defaultValue": "redacted",
+                "defaultValue": "now",
                 "lookups": {
                     "equals": {"type": "time"},
                     "gt": {"type": "time"},
@@ -928,7 +1041,14 @@ snapshots["test_query_html_no_perms context"] = {
         },
         "version": "redacted",
     },
-    "initialState": {"fields": [], "filters": [], "model": "", "results": []},
+    "initialState": {
+        "fields": [],
+        "filterErrors": [],
+        "filters": [],
+        "model": "",
+        "results": [],
+        "version": "redacted",
+    },
     "sentryDsn": None,
 }
 
@@ -1419,6 +1539,12 @@ snapshots["test_query_ctx context"] = {
                         "prettyName": "name",
                         "type": "string",
                     },
+                    "only_in_list_view": {
+                        "concrete": True,
+                        "model": "string",
+                        "prettyName": "only_in_list_view",
+                        "type": "string",
+                    },
                     "onsale": {
                         "concrete": True,
                         "model": "boolean",
@@ -1451,6 +1577,7 @@ snapshots["test_query_ctx context"] = {
                     "default_sku",
                     "is_onsale",
                     "name",
+                    "only_in_list_view",
                     "onsale",
                     "producer",
                     "size",
@@ -1510,9 +1637,67 @@ snapshots["test_query_ctx context"] = {
                         "model": None,
                         "prettyName": "count",
                         "type": "number",
-                    }
+                    },
+                    "day": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "day",
+                        "type": "number",
+                    },
+                    "hour": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "hour",
+                        "type": "number",
+                    },
+                    "minute": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "minute",
+                        "type": "number",
+                    },
+                    "month": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "month",
+                        "type": "number",
+                    },
+                    "quarter": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "quarter",
+                        "type": "number",
+                    },
+                    "second": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "second",
+                        "type": "number",
+                    },
+                    "week_day": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "week_day",
+                        "type": "number",
+                    },
+                    "year": {
+                        "concrete": True,
+                        "model": None,
+                        "prettyName": "year",
+                        "type": "number",
+                    },
                 },
-                "sortedFields": ["count"],
+                "sortedFields": [
+                    "count",
+                    "day",
+                    "hour",
+                    "minute",
+                    "month",
+                    "quarter",
+                    "second",
+                    "week_day",
+                    "year",
+                ],
             },
         },
         "baseUrl": "/data_browser/",
@@ -1601,7 +1786,7 @@ snapshots["test_query_ctx context"] = {
             },
             "time": {
                 "defaultLookup": "equals",
-                "defaultValue": "redacted",
+                "defaultValue": "now",
                 "lookups": {
                     "equals": {"type": "time"},
                     "gt": {"type": "time"},
@@ -1624,6 +1809,34 @@ snapshots["test_query_ctx context"] = {
         },
         "version": "redacted",
     },
-    "initialState": {"fields": [], "filters": [], "model": "", "results": []},
+    "initialState": {
+        "fields": [],
+        "filterErrors": [],
+        "filters": [],
+        "model": "",
+        "results": [],
+        "version": "redacted",
+    },
     "sentryDsn": None,
 }
+
+snapshots["test_query_json data"] = {
+    "fields": [
+        {"path": ["size"], "prettyPath": ["size"], "priority": 0, "sort": "dsc"},
+        {"path": ["name"], "prettyPath": ["name"], "priority": 1, "sort": "asc"},
+        {
+            "path": ["size_unit"],
+            "prettyPath": ["size_unit"],
+            "priority": None,
+            "sort": None,
+        },
+    ],
+    "filterErrors": [None, None],
+    "filters": [
+        {"lookup": "lt", "path": ["size"], "prettyPath": ["size"], "value": "2"},
+        {"lookup": "gt", "path": ["id"], "prettyPath": ["id"], "value": "0"},
+    ],
+    "model": "tests.Product",
+    "results": [[1.0, "a", "g"], [1.0, "b", "g"]],
+    "version": "redacted",
+}
