@@ -6,7 +6,6 @@ from data_browser.query import (
     ASC,
     DSC,
     BooleanFieldType,
-    BoundField,
     BoundFilter,
     BoundQuery,
     NumberFieldType,
@@ -32,66 +31,73 @@ def query():
 @pytest.fixture
 def orm_models():
     return {
+        "string": orm._get_fields_for_type(StringFieldType),
+        "number": orm._get_fields_for_type(NumberFieldType),
         "app.model": orm.OrmModel(
             fields={
-                "fa": orm.OrmField(
-                    type_=StringFieldType,
-                    concrete=True,
+                "fa": orm.OrmConcreteField(
                     model_name="app.model",
+                    name="fa",
                     pretty_name="fa",
-                ),
-                "fd": orm.OrmField(
                     type_=StringFieldType,
-                    concrete=True,
+                ),
+                "fd": orm.OrmConcreteField(
                     model_name="app.model",
+                    name="fd",
                     pretty_name="fd",
-                ),
-                "fn": orm.OrmField(
                     type_=StringFieldType,
-                    concrete=False,
-                    model_name="app.model",
-                    pretty_name="fn",
                 ),
-                "bob": orm.OrmField(
-                    type_=StringFieldType,
-                    concrete=True,
+                "fn": orm.OrmCalculatedField(
+                    model_name="app.model", name="fn", pretty_name="fn"
+                ),
+                "bob": orm.OrmConcreteField(
                     model_name="app.model",
+                    name="bob",
                     pretty_name="bob",
+                    type_=StringFieldType,
                 ),
-                "num": orm.OrmField(
-                    type_=NumberFieldType,
-                    concrete=True,
+                "num": orm.OrmConcreteField(
                     model_name="app.model",
+                    name="num",
                     pretty_name="num",
+                    type_=NumberFieldType,
+                ),
+                "tom": orm.OrmFkField(
+                    model_name="app.model",
+                    name="tom",
+                    pretty_name="tom",
+                    rel_name="app.Tom",
                 ),
             },
-            fks={"tom": orm.OrmFkField(model_name="app.Tom", pretty_name="tom")},
+            admin=True,
         ),
         "app.Tom": orm.OrmModel(
             fields={
-                "jones": orm.OrmField(
-                    type_=StringFieldType,
-                    concrete=True,
+                "jones": orm.OrmConcreteField(
                     model_name="app.Tom",
+                    name="jones",
                     pretty_name="jones",
-                )
-            },
-            fks={
+                    type_=StringFieldType,
+                ),
                 "michael": orm.OrmFkField(
-                    model_name="app.Michael", pretty_name="michael"
-                )
+                    model_name="app.Tom",
+                    name="michael",
+                    pretty_name="michael",
+                    rel_name="app.Michael",
+                ),
             },
+            admin=True,
         ),
         "app.Michael": orm.OrmModel(
             fields={
-                "bolton": orm.OrmField(
-                    type_=StringFieldType,
-                    concrete=True,
+                "bolton": orm.OrmConcreteField(
                     model_name="app.Michael",
+                    name="bolton",
                     pretty_name="bolton",
+                    type_=StringFieldType,
                 )
             },
-            fks={},
+            admin=True,
         ),
     }
 
@@ -101,15 +107,6 @@ def bound_query(query, orm_models):
     return BoundQuery(query, orm_models)
 
 
-@pytest.fixture
-def fake_orm_field():
-    class FakeOrmField:
-        type_ = StringFieldType
-        concrete = False
-
-    return FakeOrmField()
-
-
 class TestQuery:
     def test_from_request(self, query):
         q = Query.from_request(
@@ -170,55 +167,56 @@ class TestQuery:
 
 class TestBoundQuery:
     def test_fields(self, bound_query):
-        assert [f.path_str for f in bound_query.fields] == ["fa", "fd", "fn"]
-
-    def test_calculated_fields(self, bound_query):
-        assert bound_query.calculated_fields == {"fn"}
+        assert [f.path for f in bound_query.fields] == [["fa"], ["fd"], ["fn"]]
 
     def test_sort_fields(self, bound_query):
-        assert [
-            (f.path_str, f.direction, f.priority) for f in bound_query.sort_fields
-        ] == [("fd", DSC, 0), ("fa", ASC, 1)]
+        assert [(f.path, f.direction, f.priority) for f in bound_query.sort_fields] == [
+            (["fd"], DSC, 0),
+            (["fa"], ASC, 1),
+        ]
 
-    def test_filters(self, bound_query):
+    def test_filters(self, bound_query, orm_models):
+        orm_field = orm_models["app.model"].fields["bob"]
         assert bound_query.filters == [
-            BoundFilter([], "bob", None, ["bob"], "equals", "fred", StringFieldType)
+            BoundFilter(
+                orm.OrmBoundField(orm_field, orm_field, [], ["bob"]), "equals", "fred"
+            )
         ]
 
     def test_bad_field(self, orm_models):
         query = Query("app.model", [QueryField("yata")], [])
         bound_query = BoundQuery(query, orm_models)
-        assert [f.path_str for f in bound_query.fields] == []
+        assert [f.path for f in bound_query.fields] == []
 
     def test_bad_fk(self, orm_models):
         query = Query("app.model", [QueryField("yata__yata")], [])
         bound_query = BoundQuery(query, orm_models)
-        assert [f.path_str for f in bound_query.fields] == []
+        assert [f.path for f in bound_query.fields] == []
 
     def test_bad_fk_field(self, orm_models):
         query = Query("app.model", [QueryField("tom__yata")], [])
         bound_query = BoundQuery(query, orm_models)
-        assert [f.path_str for f in bound_query.fields] == []
+        assert [f.path for f in bound_query.fields] == []
 
     def test_bad_fk_field_aggregate(self, orm_models):
         query = Query("app.model", [QueryField("tom__jones__yata")], [])
         bound_query = BoundQuery(query, orm_models)
-        assert [f.path_str for f in bound_query.fields] == []
+        assert [f.path for f in bound_query.fields] == []
 
     def test_bad_long_fk(self, orm_models):
         query = Query("app.model", [QueryField("yata__yata__yata")], [])
         bound_query = BoundQuery(query, orm_models)
-        assert [f.path_str for f in bound_query.fields] == []
+        assert [f.path for f in bound_query.fields] == []
 
     def test_aggregate(self, orm_models):
         query = Query("app.model", [QueryField("tom__jones__count")], [])
         bound_query = BoundQuery(query, orm_models)
-        assert [f.path_str for f in bound_query.fields] == ["tom__jones__count"]
+        assert [f.path for f in bound_query.fields] == [["tom", "jones", "count"]]
 
     def test_bad_filter(self, orm_models):
         query = Query("app.model", [], [QueryFilter("yata", "equals", "fred")])
         bound_query = BoundQuery(query, orm_models)
-        assert [f.path_str for f in bound_query.fields] == []
+        assert [f.path for f in bound_query.fields] == []
 
     def test_bad_filter_value(self, orm_models):
         query = Query(
@@ -230,73 +228,10 @@ class TestBoundQuery:
         assert [f.value for f in bound_query.filters] == ["fred", 1]
         assert [f.value for f in bound_query.valid_filters] == [1]
 
-
-class TestBoundField:
-    def test_path_properties(self, fake_orm_field):
-        bf = BoundField(
-            ["bob", "fred"],
-            "joe",
-            "max",
-            ["bob", "fred", "joe"],
-            None,
-            None,
-            fake_orm_field,
-        )
-        assert bf.model_path_str == "bob__fred"
-        assert bf.field_path_str == "bob__fred__joe"
-        assert bf.path_str == "bob__fred__joe__max"
-
-        bf = BoundField([], "joe", "max", ["joe"], None, None, fake_orm_field)
-        assert bf.model_path_str == ""
-        assert bf.field_path_str == "joe"
-        assert bf.path_str == "joe__max"
-
-        bf = BoundField(
-            ["bob", "fred"],
-            "joe",
-            None,
-            ["bob", "fred", "joe"],
-            None,
-            None,
-            fake_orm_field,
-        )
-        assert bf.model_path_str == "bob__fred"
-        assert bf.field_path_str == "bob__fred__joe"
-        assert bf.path_str == "bob__fred__joe"
-
-
-class TestBoundFilter:
-    def test_path_properties(self):
-        bf = BoundFilter(
-            ["bob", "fred"],
-            "joe",
-            "max",
-            ["bob", "fred", "joe"],
-            "gt",
-            5,
-            StringFieldType,
-        )
-        assert bf.model_path_str == "bob__fred"
-        assert bf.field_path_str == "bob__fred__joe"
-        assert bf.path_str == "bob__fred__joe__max"
-
-        bf = BoundFilter([], "joe", "max", ["joe"], "gt", 5, StringFieldType)
-        assert bf.model_path_str == ""
-        assert bf.field_path_str == "joe"
-        assert bf.path_str == "joe__max"
-
-        bf = BoundFilter(
-            ["bob", "fred"],
-            "joe",
-            None,
-            ["bob", "fred", "joe"],
-            "gt",
-            5,
-            StringFieldType,
-        )
-        assert bf.model_path_str == "bob__fred"
-        assert bf.field_path_str == "bob__fred__joe"
-        assert bf.path_str == "bob__fred__joe"
+    def test_fk(self, orm_models):
+        query = Query("app.model", [QueryField("tom")], [])
+        bound_query = BoundQuery(query, orm_models)
+        assert [f.path for f in bound_query.fields] == []
 
 
 class TestFieldType:
@@ -306,12 +241,10 @@ class TestFieldType:
 
 class TestStringFieldType:
     def test_validate(self):
-        assert BoundFilter(
-            [], "bob", None, ["bob"], "contains", "hello", StringFieldType
-        ).is_valid
-        assert not BoundFilter(
-            [], "bob", None, ["bob"], "pontains", "hello", StringFieldType
-        ).is_valid
+        orm_field = orm.OrmConcreteField("", "bob", "bob", StringFieldType)
+        orm_bound_field = orm.OrmBoundField(orm_field, orm_field.type_, [], ["bob"])
+        assert BoundFilter(orm_bound_field, "contains", "hello").is_valid
+        assert not BoundFilter(orm_bound_field, "pontains", "hello").is_valid
 
     def test_default_lookup(self):
         assert StringFieldType.default_lookup == "equals"
@@ -322,21 +255,13 @@ class TestStringFieldType:
 
 class TestNumberFieldType:
     def test_validate(self):
-        assert BoundFilter(
-            [], "bob", None, ["bob"], "gt", "6.1", NumberFieldType
-        ).is_valid
-        assert not BoundFilter(
-            [], "bob", None, ["bob"], "pontains", "6.1", NumberFieldType
-        ).is_valid
-        assert not BoundFilter(
-            [], "bob", None, ["bob"], "gt", "hello", NumberFieldType
-        ).is_valid
-        assert BoundFilter(
-            [], "bob", None, ["bob"], "is_null", "True", NumberFieldType
-        ).is_valid
-        assert not BoundFilter(
-            [], "bob", None, ["bob"], "is_null", "hello", NumberFieldType
-        ).is_valid
+        orm_field = orm.OrmConcreteField("", "bob", "bob", NumberFieldType)
+        orm_bound_field = orm.OrmBoundField(orm_field, orm_field.type_, [], ["bob"])
+        assert BoundFilter(orm_bound_field, "gt", "6.1").is_valid
+        assert not BoundFilter(orm_bound_field, "pontains", "6.1").is_valid
+        assert not BoundFilter(orm_bound_field, "gt", "hello").is_valid
+        assert BoundFilter(orm_bound_field, "is_null", "True").is_valid
+        assert not BoundFilter(orm_bound_field, "is_null", "hello").is_valid
 
     def test_default_lookup(self):
         assert NumberFieldType.default_lookup == "equals"
@@ -347,21 +272,16 @@ class TestNumberFieldType:
 
 class TestTimeFieldType:
     def test_validate(self):
-        assert BoundFilter(
-            [], "bob", None, ["bob"], "gt", "2018-03-20T22:31:23", TimeFieldType
-        ).is_valid
+        orm_field = orm.OrmConcreteField("", "bob", "bob", TimeFieldType)
+        orm_bound_field = orm.OrmBoundField(orm_field, orm_field.type_, [], ["bob"])
+        assert BoundFilter(orm_bound_field, "gt", "2018-03-20T22:31:23").is_valid
+        assert not BoundFilter(orm_bound_field, "gt", "hello").is_valid
         assert not BoundFilter(
-            [], "bob", None, ["bob"], "gt", "hello", TimeFieldType
-        ).is_valid
-        assert not BoundFilter(
-            [], "bob", None, ["bob"], "pontains", "2018-03-20T22:31:23", TimeFieldType
-        ).is_valid
-        assert BoundFilter(
-            [], "bob", None, ["bob"], "is_null", "True", TimeFieldType
-        ).is_valid
-        assert not BoundFilter(
-            [], "bob", None, ["bob"], "is_null", "hello", TimeFieldType
+            orm_bound_field, "pontains", "2018-03-20T22:31:23"
         ).is_valid
+        assert BoundFilter(orm_bound_field, "is_null", "True").is_valid
+        assert not BoundFilter(orm_bound_field, "is_null", "hello").is_valid
+        assert BoundFilter(orm_bound_field, "gt", "now").is_valid
 
     def test_default_lookup(self):
         assert TimeFieldType.default_lookup == "equals"
@@ -375,18 +295,12 @@ class TestTimeFieldType:
 
 class TestBooleanFieldType:
     def test_validate(self):
-        assert BoundFilter(
-            [], "bob", None, ["bob"], "equals", "True", BooleanFieldType
-        ).is_valid
-        assert BoundFilter(
-            [], "bob", None, ["bob"], "equals", "False", BooleanFieldType
-        ).is_valid
-        assert not BoundFilter(
-            [], "bob", None, ["bob"], "equals", "hello", BooleanFieldType
-        ).is_valid
-        assert not BoundFilter(
-            [], "bob", None, ["bob"], "pontains", "True", BooleanFieldType
-        ).is_valid
+        orm_field = orm.OrmConcreteField("", "bob", "bob", BooleanFieldType)
+        orm_bound_field = orm.OrmBoundField(orm_field, orm_field.type_, [], ["bob"])
+        assert BoundFilter(orm_bound_field, "equals", "True").is_valid
+        assert BoundFilter(orm_bound_field, "equals", "False").is_valid
+        assert not BoundFilter(orm_bound_field, "equals", "hello").is_valid
+        assert not BoundFilter(orm_bound_field, "pontains", "True").is_valid
 
     def test_default_lookup(self):
         assert BooleanFieldType.default_lookup == "equals"
